<?php

/*
 *	Article System
 *	Online journal management tool written in PHP/MySQL/PostgreSQL.
 *	This code is available at http://sourceforge.net/projects/artsys
 *
 *	Copyright (C) 2001-2005 Jan Hnatek
 *	Distributed under the terms of the GNU General Public License
 *
 *	Date:	  05/08/2005
 *	Version:  0.6   	
 */

###############################################################################
# form_process : form abstraction

// package artsys.forms

/**
 *	An abstract class implementing generic functionality for processing user's input.
 *	
 *	This class encapsulates generic functions for working with data coming from user forms. 
 *	Descendants must only override certain functions that perform context-specific tasks, 
 *	like custom checking of data, storing correct data, etc.<br>
 *	based on Yuri Makassiouk's (&lt;racer@bfpg.ru&gt;) FormProcessor
 *	
 *	@author	Jan Hnatek
 *	@package forms
 */
class form_process {
//----------------//

	// VARIABLES
	//
	
	/**	Name of the form. */
    var $name;
    
	/**	Section object this form belongs to. */
    var $section;
	
	/**	'True' if form is displayed for the first time.
     *	This value is set in the class constructor. 
	 *	If form finds its fields among values received from a POST, then it decides it was displayed 
	 *	before and submitted.
     */
	var $first_time;
	
    /**	An associative array holding values of form's controls. */
	var $values;
	
    /**	An array that holds error messages.
     *	This array should be filled by member functions that peform checking of user's input. 
	 *	It is used to make a descision weather form can be considered completed or not.
     */
	var $errors = array();
	
    /**	Error messages format.
     *	You must set this value according to the appearance of error messages that 
	 *	would suite your application's needs
     */
	var $error_message_format = '<br><span class="error">%s</span>';
	
    /**	Prefix that class uses to build names of HTML form elements.
     *	A string with only one special thing about it: it should be unique enough to not get 
	 *	names of controls that are managed by this form confused with names of other HTML elements.
     */
	var $field_prefix = 'form_data_';
	
	/**	Wizard object that this form is attached to.
	 *	Wizard is a sequence of form_process objects.
	 */
	var $wizard;

    
	// CONSTRUCTOR
	//
	/**	Constructor of the class.
     *	Initializes variables and 'smells the air' deciding what state the form is in. 
	 *	If it can find values that correspond to this form among global variables,
	 *	it assumes that the form was submitted at least once, which means that values must be checked
	 *	and some further decisions made.
     *
     *	@param string name Name of the form
     *	@param reference wizard A (reference to) wizard object this form is attached to
	 */
	function form_process (&$section, $name, $wizard=0) {
		debug ("constructor: $name");
		$this->section = $section;
		$this->right = $section->get_right ($name);
		$this->name = $name;
		$this->wizard = $wizard;
		//debug ("my wizard: ". $this->wizard->name);
		
		// paranoia: double check that form won't be displayed if alert_run fails
		if (!$this->prepare ())
			return false;

		if (isset ($_POST[$this->field_prefix.$this->name]))
			$this->values = $_POST[$this->field_prefix.$this->name];
		$this->first_time = (count($this->values) == 0);
		
		if (!$this->first_time)
			$this->custom_check ();
		
		if ($this->is_completed ()) {
			//debug ($this->name." completed");
			$this->store_data();
			if ($this->wizard != 0) {
				$this->wizard->advance ();
				// TEMP
				//$this->display_form();
			}
		} else
			$this->display_form();
	}
    
	
	// METHODS
	//
	
	/**	Return access right according to ini.
	 *	If not set in ini, use the right of this object.
	 *	That makes it a tree structure of rights, where leaves have highest priority.
	 */
	function get_right ($path) {
		$path = "{$this->name}:$path";
		//debug ("form_process->get_right(:$path)");
		return $this->section->isset_right ($path) ? $this->section->get_right ($path) : $this->right;
	}
	
    /**	Returns true if submitted data successfully passed verification.
     *	@return bool
     *	@access private
     */
	function is_completed () {
		return (!$this->first_time && count ($this->errors) <= 0);
	}
    
    /**	Sets up initial data. Called from constructor.
     *	This abstract function must be overridden in sub-classes to implement application logic.
	 *	Overridden functions must write error messages in errors array, if a check fails.
     *	@abstract
     */
	function prepare () {
		//abstract
	}
	
    /**	Performs custom checking of data.
     *	This abstract function must be overridden in sub-classes to implement application logic.
	 *	Overridden functions must write error messages in errors array, if a check fails.
     *	@abstract
     */
	function custom_check () {
		//abstract
	}


    /**	Displays form's html.
     *	This abstract function must be overriden in sub-classes to implement actual form's displaying.
     *	@abstract
     */
    function display_form () {
    	//abstract
	}
	
    /**	Stores user's input in the application.
     *	This abstract function must be overriden in sub-classes to implement application logic. 
	 *	It is called if form is considered completed in order to save user's input in a database or otherwise.
     *	@abstract
     */
	function store_data () {
		//abstract
	}
	
    /**	Sets values of properties.
     *	Access method. Currently it is just a trivial assigning of a value to a local variable, 
	 *	but can be extended to do something useful.
     *
     *	@param string name Name of the object's field to be set.
     *	@param string value Value to be assigned to the field.
     */
	function set ($name, $value) {
		$this->$name = $value;
	}
	
    /**	Assigns to local error[$name] variable.
     *	It should be called from custom_check. This is the ontrol point of whether use alert() or not.
     *	@param string name Name of data element.
	 *	@param string text Text of the error message.
     */
	function error ($name, $text) {
		// alert type check (dialog with OK button)
		//$this->errors[$name] = alert ($text);
		// message type check
		$this->errors[$name] = $text;
	}
	
    /**	Prints formatted error message.
     *	Applications should call this function at a 'placeholder' that displays an error message if error occurs.
     *	@param string name Name of data element.
     */
	function error_report ($name) {
		if (isset ($this->errors[$name]))
			printf ($this->error_message_format, $this->errors[$name]);
	}
	
	/** Returns initial value of a local variable
	 *	Returns false, if there is no such variable.
	 */
	function get_initial_value ($name) {
		if (isset ($this->values[$name]))
			return $this->values[$name];
		else
			return false;
	}
	
	/** Echoes initial value of a local variable to output.
	 *	It's done by calling get_initial_value.
	 *	@return	void
	 *	@access	public
	 *	@see	get_initial_value
	 */
	function initial_value ($name) {
		echo $this->get_initial_value ($name);
	}
	
	/**	Returns 'parent' wizard object (used for wizard sequence links etc.)
	 */
	function get_wizard () {
		return $this->wizard;
	}
	
	
	// METHODS : PROTECTED
	//
	
	/**	Default form item rendering method.
	 */
	function _display_item (&$t) {
		if ($t->visible ()) { ?>
			<tr><td><?	$t->description () ?></td>
				<td><?	$t->display ();
						$this->error_report ($t->name);	?></td>
			</tr>
	<? 	}
	}
	
	/**	Tool for simpler data checking within <code>custom_check</code>.
   	 *	@see custom_check
	 */
	function _nonempty ($name, $message) {
		// FIXME: allow 0 as a value! (if it's set);
		if (empty ($this->values[$name]))
		// TODO: "0" too?
		//	if (!$this->values[$name] === 0)
				$this->error ($name, $message);
	}
	
	/**	Since unset CheckBox form items don't return value, this is a simple
	 *	way to initialize them. Sets all given values of $this->values to "off".
   	 *	@param	scalar/array list Value or array of values to set to zero.
	 */
	function _init_checkboxes ($list) {
		if (is_array ($list)) {
			foreach ($list as $val)
				if (!isset ($this->values[$val]))
					$this->values[$val] = "off";
		} else
			if (!isset ($this->values[$val]))
				$this->values[$list] = "off";
	}
	
}	// class form_process

?>
