<?php

/*
 *	Article System
 *	Online journal management tool written in PHP/MySQL/PostgreSQL.
 *	This code is available at http://sourceforge.net/projects/artsys
 *
 *	Copyright (C) 2001-2005 Jan Hnatek
 *	Distributed under the terms of the GNU General Public License
 *
 *	Date:	  05/08/2005
 *	Version:  0.6   	
 */

###############################################################################
# SEARCH ENGINE
#


/**	Fulltext search using pre-built database word index.<br>
 *	Word frequency is taken into account. 
 *	Stopwords are implemented with selectable limit percentage.<br>
 *	Uses global <code>$db</code> object.

 DONE:
	* possible limitations (unpublished articles ...)
	-> post search filter
	* more items for index (content, title, subtitle)
	-> array of columns
 TODO:
 	* should use some html_rewriter class in the future
	  (for now just global strip_html_tags function)
	  see php strip_html_tags function!
	* index build limits .. word length (>1), cut non-word chars more precisely?
	* grep -n tool .. possibly with word highlight (low priority)
 *
 *	@author	Jan Hnatek
 */

class fulltext_index {
//------------------//

	// VARIABLES
	//
	
	/** Database object (table) name. */
	var $db_object;
	/** Array of database item (column) names. */
	var $db_items;
	/**	Stopword limit percentage. */
	var $stopword_pct;
	/**	Minimal word length. */
	var $min_wordlen;
	
	
	// CONSTRUCTOR
	//
	
	/**	Set the database table name and columns used in word index.
	 *	@param	$object	Database object, table name.
	 *	@param	$items	Array of table columns to merge for indexing.
	 *	@access	public
	 */
	function fulltext_index ($object, $items) {
		$this->db_object = $object;
		$this->db_items = is_array ($items) ? $items : array ($items);
		// search words can at most be present in 70% of the objects (by default)
		$this->stopword_pct = 0.7;
		// minimal word length is 3 characters
		$this->min_wordlen = 3;
	}
		
	
	// METHODS
	//
	
	/**	Change the default search settings.
	 *	@param	$stopword_pct	Stop word percentage (default 0.7).
	 *	@param	$min_wordlen	Minimal word length (default 3).
	 */
	function set_defaults ($stopword_pct, $min_wordlen) {
		$this->stopword_pct = $stopword_pct;
		$this->min_wordlen = $min_wordlen;
	}
	
	/** Get data for item with given id.
	 *	@access	public
	 */
	function get_item_data ($item_id) {
		global $db;
		$list_cols = array_reduce ($this->db_items, "string_reduce");
		$RS = $db->exec ("SELECT $list_cols FROM $this->db_object WHERE id='$item_id'", 1);
		$tmp = $db->get_result_array ($db->NUM);
		if (!$tmp)
			return false;
		else {
			$ret = "";
			foreach ($tmp as $data)
				$ret .= strip_html_tags ($data);
			return $ret;
		}
	}
	
	/** Build word index for one item.
	 *	@access	public
	 */
	function create_item_index ($item_id) {
		global $db;

		// test if this item was already indexed
		$db->exec ("SELECT id FROM object_word_link WHERE object_id='$item_id'");
		if ($db->get_result ())
			return false;
		
		// get index text
		if (!$index_text = $this->get_item_data ($item_id))
			return false;
	
		// convert non-word characters to whitespace and remove multiple whitespace
		$search = array ("/\./", "/,/", "/\"/", "/\n/", "/\r/", "/'/", "(\s+)"); // "/\(/", "/\)/",
		$replace = array (" ",   " ",   " ",    " ",    " ",    "\'",  " "    );
		$index_text = preg_replace ($search, $replace, $index_text);
		
		// split text on whitespace
		$index_array =& split (" ", $index_text);
		// count the total words in index text
		$total_word_count = count ($index_array);
		// count the number of instances of each word
		$word_count_array = array_count_values ($index_array);
		// strip double words
		$index_array = array_unique ($index_array);
		// count unique words
		$unique_word_count = count ($index_array);

		// TODO: remove '' ? or temporary info to some external window
		// with immediate write (no tables); like this it's unusable
		//echo "indexing: ", $item_id, "<br>";
		foreach ($index_array as $index_word) {
			if (strlen ($index_word) >= $this->min_wordlen) {
				$RS = $db->exec ("SELECT * FROM word WHERE word='$index_word'");
				$word_rs = array ();
				while ($tmp =& $db->get_result_array ())
					array_push ($word_rs, $tmp);
				if (count ($word_rs) == 1) {
					$word_id = $word_rs[0]['id'];
					$db->exec ("UPDATE word SET object_count=(object_count+1) WHERE id='$word_id'");
				} else {
					$db->exec ("INSERT INTO word (word, object_count) VALUES ('$index_word', '1')");
					// postgres sequence as parameter (mysql doesn't care)
					$word_id = $db->last_insert_id ("word_id_seq");
				}

				// calculate the relevance ranking for this word
				$frequency = $word_count_array[$index_word] / $total_word_count;
				$db->exec ("INSERT INTO object_word_link (word_id, object_id, frequency)
							VALUES ('$word_id', '$item_id', '$frequency')");
				//echo "$index_word ($frequency) ";			
			}
		}
	}
	
	/** Remove index data for one item (undo <code>create_item_index</code>).
	 *	@access	public
	 */
	function remove_item_index ($item_id) {
		global $db;
		// get all words linked to given object
		$db->exec ("SELECT id, word_id FROM object_word_link WHERE object_id='$item_id'");
		// TODO: rewrite with array_exec		
		$words = array ();
		while ($tmp =& $db->get_result_array ()) {
			array_push ($words, $tmp["word_id"]);
		}
		// decrement word counters or remove unused words
		foreach ($words as $word_id) {
			$db->exec ("SELECT object_count FROM word WHERE id='$word_id'");
			$tmp =& $db->get_result_array ();
			if ($tmp["object_count"] == 1) {
				$db->exec ("DELETE FROM word WHERE id='$word_id'");
			} else {
				$db->exec ("UPDATE word SET object_count=(object_count-1) WHERE id='$word_id'");
			}
		}
		// delete all object links
		$db->exec ("DELETE FROM object_word_link WHERE object_id='$item_id'");
	}
	
	/** Clear all database index data.
	 *	@access	public
	 */
	function remove_all_indexes () {
		global $db;
		$db->exec ("DELETE FROM word;");
		$db->exec ("DELETE FROM object_word_link;");
	}
	
	/**	Build whole index. Supposedly VERY (CPU) time consuming.
	 */
	function rebuild_all_indexes () {
		global $db;
		$this->remove_all_indexes ();
		// TODO: limitations for object -> index only those visible online !!
		$db->exec ("SELECT max(id) FROM $this->db_object");
		$tmp =& $db->get_result_array($db->NUM);
		// $tmp[0] is max_id
		for ($id=1; $id <= $tmp[0]; $id++)
			$this->create_item_index ($id);
	}
	
	/**	Perform search for given words.
	 *	@param	$text	String (multiple space-delimited words) to find.
	 *	@param	$exact	Boolean, true to perform word-exact match.
	 *	@param	$case_sensitive	Boolean, true to enable case sensitive word matching.
	 *	@access	public
	 */
	function find ($text, $exact=false, $case_sensitive=false) {
		global $db;
		
    	// strip whitespace & split on whitespace
		$text = preg_replace ("(\s+)", " ", $text);
    	$search_array =& split (" ", $text);	
		
		// get the total number of objects
		$db->exec ("SELECT COUNT(*) AS count FROM $this->db_object");
		$object_count = array ();
		while ($tmp =& $db->get_result_array())
			array_push ($object_count, $tmp);
		$total_object_count = $object_count[0]['count'];

		$word_sql = "";
		$i = 0;
		// build the word query string
		foreach ($search_array as $search_word) {
			if ($i++ > 0)
				$word_sql .= " OR ";
			// note: no space after LOWER function call in MySQL!
			$word_sql .= $case_sensitive ? "word.word" : "LOWER(word.word)";
			$word_sql .= $exact ? 
				"='" .strToLower ($search_word) ."' " : 
				" LIKE '%" .strToLower ($search_word) ."%' ";
		}

		// build the full search query using logical OR if multiple words are searched on
		$query = "SELECT $this->db_object.id, object_word_link.frequency
					FROM $this->db_object, object_word_link, word
					WHERE ($this->db_object.id = object_word_link.object_id)
					AND (word.id = object_word_link.word_id)
					AND ($word_sql)
					AND ((word.object_count / $total_object_count) < $this->stopword_pct)
					ORDER BY object_word_link.frequency DESC";
		// FIXME: GROUP BY $this->db_object.id
		// makes mess in ordering ? should be FIRST ORDER, then GROUP BY
		if ($db->exec ($query)) {		
			$rs = array ();
			while ($tmp =& $db->get_result_array ())
				array_push ($rs, $tmp["id"]);
			return array_unique ($rs);
		} else
			return false;
	}
	
}	// class fulltext_search

/*
	TODO: see MySQL doc !!!!
	seems to be MySQL specific, though


As of Version 3.23.23, MySQL has support for full-text indexing and searching. 
Full-text indexes in MySQL are an index of type FULLTEXT. FULLTEXT indexes are used with MyISAM tables
and can be created from CHAR, VARCHAR, or TEXT columns at CREATE TABLE time or added later with ALTER
TABLE or CREATE INDEX. For large datasets, it will be much faster to load your data into a table that has
no FULLTEXT index, then create the index with ALTER TABLE (or CREATE INDEX). Loading data into a table
that already has a FULLTEXT index will be slower.

Full-text searching is performed with the MATCH() function.

mysql> CREATE TABLE articles (
    ->   id INT UNSIGNED AUTO_INCREMENT NOT NULL PRIMARY KEY,
    ->   title VARCHAR(200),
    ->   body TEXT,
    ->   FULLTEXT (title,body)
    -> );
Query OK, 0 rows affected (0.00 sec)

mysql> INSERT INTO articles VALUES
    -> (NULL,'MySQL Tutorial', 'DBMS stands for DataBase ...'),
    -> (NULL,'How To Use MySQL Efficiently', 'After you went through a ...'),
    -> (NULL,'Optimising MySQL','In this tutorial we will show ...'),
    -> (NULL,'1001 MySQL Tricks','1. Never run mysqld as root. 2. ...'),
    -> (NULL,'MySQL vs. YourSQL', 'In the following database comparison ...'),
    -> (NULL,'MySQL Security', 'When configured properly, MySQL ...');
Query OK, 6 rows affected (0.00 sec)
Records: 6  Duplicates: 0  Warnings: 0

mysql> SELECT * FROM articles
    ->          WHERE MATCH (title,body) AGAINST ('database');
+----+-------------------+------------------------------------------+
| id | title             | body                                     |
+----+-------------------+------------------------------------------+
|  5 | MySQL vs. YourSQL | In the following database comparison ... |
|  1 | MySQL Tutorial    | DBMS stands for DataBase ...             |
+----+-------------------+------------------------------------------+
2 rows in set (0.00 sec)

*/
?>

