<?php

/*
 *	Article System
 *	Online journal management tool written in PHP/MySQL/PostgreSQL.
 *	This code is available at http://sourceforge.net/projects/artsys
 *
 *	Copyright (C) 2001-2005 Jan Hnatek
 *	Distributed under the terms of the GNU General Public License
 *
 *	Date:	  05/08/2005
 *	Version:  0.6   	
 */

###############################################################################

/**
 *	Database abstraction class - parent to specific database handlers.<br>
 *
 *	@author	Jan Hnatek
 *	@package db
 */

class abstract_db {
//---------------//

	// VARIABLES
	//

	/**	Connection variables. See constructor. */
	var $name;
	var $host;
	var $user;
	var $password;

	/**	Connection. */
	var $conn;
	/**	Persistent connection flag. */
	var $persistent;
	/**	Result resource. */
	var $rs;

	/**	The default desired array result type ('NUM'eric, 'ASSOC'iative, 'BOTH'). */
	var $_array_return_type;
	
	/** Array result types : 'NUM'eric, 'ASSOC'iative, 'BOTH'.
	 *	Should be set in subclasses' constructor.
	 */
	var $NUM;
	var $ASSOC;
	var $BOTH;	

	// TODO: ?
	//var $dateformat;


	// CONSTRUCTOR
	//

	/** Basic database constructor. Sets up database connection parameters.
	 *	Can be called from subclasses. Subclasses are also meant to set up
	 *	<code>$NUM</code>, <code>$ASSOC</code> and <code>$BOTH</code> variables
	 *	(the type of array with <code>get_result_array</code>).
	 *	@param	$name	Name of the database to connect to.
	 *	@param	$host	Hostname.
	 *	@param	$user	Name of the database user.
	 *	@param	$passwd	$user's database password.
	 */
	function abstract_db ($name, $host, $user, $passwd) {
		$this->name = $name;
		$this->host = $host;
		$this->user = $user;
		$this->password = $passwd;
	}


	// GENERAL METHODS
	//

	/**	Return information string depending on success of last query.
	 *	@param	$msg_fail	Optional string to describe the error, if any. 
	 *						Comes with detailed information from db.
	 *	@param	$msg_ok		Optional string to return, when query was succesfully done.
	 *	@access	public
	 */
	function report ($msg_fail="", $msg_ok="") {
		if (!$this->rs)
			return msg("db error")." (".$this->errno().") : ".$this->error()."<br>".$msg_fail."<br>";
		else
			return $msg_ok;
	}

	/** Execute a query and return the results in an array.
	 *	A shortcut for an often used command set - a combination
	 *	of exec and get_result_array.
	 *	@param	$query			The text of the query.
	 *	@param	$result_type	Array return type.	
	 *	@returns array
	 *	@see exec, get_result_array
	 */
	function array_exec ($query, $result_type = -1) {
		if ($result_type == -1)
			$result_type = $this->_array_return_type;
		$RS = $this->exec ($query);
		if ($RS) {
			$res = array ();
			while ($line = $this->get_result_array ($result_type))
				array_push ($res, $line);
			return $res;
		} else
			return false;
	}		

	/** Another name for <code>exec</code>. An expected function name, too.
	 *	@see exec
	 */
	function query ($query) {
		return $this->exec ($query);
	}

	/** Another name for <code>array_exec</code>.
	 *	@see array_exec
	 */
	function array_query ($query, $result_type = -1) {
		return $this->array_exec ($query, $result_type);
	}

	/**	Debug function. Calls external handler.
	 *	@param	$query		The text of the query.
	 *	@param	$boolean	Integer, switch to enable per-query debug output.
 	 *	@access	public
	 */
	function debug ($query, $bool) {
		if ($bool)
			debug_db ($query);
		else
			debug_db_massive ($query);
	}

	/**	Reports error if condition is false. Calls external handler.
 	 *	@access	public
	 */
	function conderror ($cond, $text, $file='') {
		conderror ($cond, $text, $file);
	}


	// ABSTRACT METHODS
	//

	/**	Open a database connection.
	 */
	function open ($persist=0) {
		// abstract
	}

	/**	Close database connection.
	 */
	function close () {
		// abstract
	}

	/**	Execute a database query. Store and return its result.
	 */
	function exec ($query) {
		// abstract
	}

	/**	Return number of rows in result
	 */
	function num_rows () {
		// abstract
	}


	// ABSTRACT WRAPPERS
	//

	/**	Wrapper, return error number.
	 */
	function errno () {
		// abstract
	}

	/**	Wrapper, return textual error message.
	 */
	function error () {
		// abstract
	}

	/**	Wrapper, return a result row as an enumerated array or FALSE.
	 */
	function get_result () {
		// abstract
	}

	/**	Wrapper, return a result row as an associative array, a numeric array, or both.
	 *	The default return array type is set in constructor or with <code>set_array_return_type</code>.
	 */
	function get_result_array () {
		// abstract
	}

	/**	Wrapper, return a result row as an object.<br>
	 */
	function get_result_object () {
		// abstract
	}

	/**	Wrapper, return number of affected records in previous operation.
	 */
	function get_affected_rows () {
		// abstract
	}

	/**	Wrapper, return the object id generated from the previous INSERT operation.
	 */
	function last_insert_id ($sequence) {
		// abstract
	}

	/**	Return uppercase identifier that this database constants start with.
	 */
	function get_db_id () {
		// abstract
  	}

	/** Returns status of DB connection (true if connected).
	 */
	function connected () {
		// abstract
	}
	
	/**	Select the way the array data is returned.
	 *	Possible solutions are "NUM" for numeric, "ASSOC" for associative and "BOTH".
	 *  @param	$type	one of "NUM", "ASSOC", "BOTH" strings
	 *	@access	public
	 */
	function set_array_return_type ($type) {
		// abstract
  	}

} // class abstract_db

?>
