<?php

/*
 *	Article System
 *	Online journal management tool written in PHP/MySQL/PostgreSQL.
 *	This code is available at http://sourceforge.net/projects/artsys
 *
 *	Copyright (C) 2001-2005 Jan Hnatek
 *	Distributed under the terms of the GNU General Public License
 *
 *	Date:	  05/08/2005
 *	Version:  0.6   	
 */

###############################################################################

/**
 *	An object to work with PostgreSQL database.<br>
 *	Includes functions to connect, exec queries, return results, close...
 *
 *	@author	Jan Hnatek
 *	@package db
 */

class postgres_db extends abstract_db {
//-----------------------------------//


	// VARIABLES
	//

	/**	Row counter for fetch() calls. */
	var $row;
	/**	Limit for returning result rows. */
	var $maxrow;

	/**	Compatibility setting (PHP 4.1 -> 4.2). */
	var $compat_mode;


	// CONSTRUCTOR
	//

	/** Calls <code>abstract_db</code> contructor.
	 *	Sets up array return type to <code>PGSQL_ASSOC</code> by default.
	 */
	function postgres_db ($iname, $ihost, $iuser, $ipasswd, $array_return_type="ASSOC") {
		abstract_db::abstract_db ($iname, $ihost, $iuser, $ipasswd);
		$this->NUM = PGSQL_NUM;
		$this->ASSOC = PGSQL_ASSOC;
		$this->BOTH = PGSQL_BOTH;
		$this->set_array_return_type ($array_return_type);
		$tmp = phpversion ();
		// TODO: a bit misleading selection (5.1 ? :-)
		$this->compat_mode = $tmp[2] == "1" ? true : false;
	}


	// METHODS
	//

	/**	Open a database connection.
	 *	@param $persist Say whether open a persistent database connection or not (1/0).
	 *	@access	public
	 *	@return	DB connection object
	 */
	function open ($persist=0) {
		$this->persistent = $persist;
		$this->conderror ($this->user, "postgres_db->open: database user not specified");
		$connstr = "host=".$this->host." dbname=".$this->name." user=".$this->user." password=".$this->password;
		if ($this->persistent == 1)
			$this->conn = pg_pconnect ($connstr);
		else
			$this->conn = pg_connect ($connstr);
		$this->conderror ($this->conn, "postgres_db->open: database connection failed");
		return $this->conn;
	}

	/**	Close database connection.
	 *	@access	public
	 */
	function close () {
		if (is_resource($this->rs)) {
			$this->compat_mode ? pg_freeresult ($this->rs) : pg_free_result ($this->rs);
		}
		pg_close ();
	}

	/**	Execute a database query. Store and return its result.
	 *	Add *any* value as the second parameter to display debug information on this particular query.
	 *	@access	public
	 *	@return query result or false when query failed
	 */
	function exec ($query) {
		$this->conderror ($query, "postgres_db->exec: no query given");
		// send number of arguments -1 to debug fn: per-query debug
		$tmp = func_num_args() - 1;
		$this->debug ($query, $tmp);
		$this->rs = $this->compat_mode ?
			pg_exec ($this->conn, $query) : pg_query ($this->conn, $query);
		$this->row = 0;
		$this->maxrow = $this->num_rows ();
		return $this->rs;
	}

	/**	Return number of rows in result
	 *	@access	public
	 */
	function num_rows () {
		// num_rows (-1 on error, 0 false too; non-zero OK)
		$x = ($this->rs) ? ($this->compat_mode ? pg_numrows ($this->rs) : pg_num_rows ($this->rs)) : 0;
		if ($x == -1) {
			$this->debug ("postgres_db->num_rows: failed to get result (result == -1)");
			$x = 0;
		}
		return $x;
	}


	// WRAPPERS
	//

	/**	Wrapper, return last error number (<code>mysql_errno()</code> in mysql).
	 *	// TODO: function not available in postgres ?
	 *	@access	public
	 */
	function errno () {
		//return "function not available in postgres";
		return "";
	}

	/**	Wrapper, return result of <code>pg_result_error()</code>.<br>
	 *	Get error message associated with result.
	 *	@access	public
	 */
	function error () {
		// FIXME: result_error complains about boolean $this->rs .. but it's error, ???
		//return $this->compat_mode ? pg_errormessage ($this->conn) : pg_result_error ($this->rs);
		return pg_errormessage ($this->conn);
	}

	/**	Wrapper, return a result row as an enumerated array or FALSE.
	 *	@access	public
	 */
	function get_result () {
		return ($this->row < $this->maxrow) ? pg_fetch_row ($this->rs, $this->row++) : false;
	}

	/**	Wrapper, return a result row as an associative array, a numeric array, or both.<br>
	 *	@param	$result_type	Optional parameter that controls how return value is initialized.
	 *	<code>result_type</code> is a constant and can take the following values: PGSQL_ASSOC, PGSQL_NUM, and PGSQL_BOTH
	 *	@access	public
	 */
	function get_result_array ($result_type = -1) {
		if ($result_type == -1)
			$result_type = $this->_array_return_type;
		return ($this->row < $this->maxrow) ? pg_fetch_array ($this->rs, $this->row++, $result_type) : false;
	}

	/**	Wrapper, return a result row as an object.<br>
	 *	@access	public
	 */
	function get_result_object () {
		return ($this->row < $this->maxrow) ? pg_fetch_object ($this->rs, $this->row++) : false;
	}

	/**	Wrapper, return number of affected records in previous operation.
	 *	@access	public
	 */
	function get_affected_rows () {
		return $this->compat_mode ? pg_cmdtuples ($this->rs) : pg_affected_rows ($this->rs);
	}

	/**	Wrapper, return the object id generated from the previous INSERT operation.
	 *	@param	$sequence	Name of the sequence used for ids in table from last INSERT query.
	 *	@access	public
	 */
	function last_insert_id ($sequence) {
		$tmp_query = "SELECT last_value FROM $sequence;";
		$tmp_rs = $this->compat_mode ?
			pg_exec ($this->conn, $tmp_query) : pg_query ($this->conn, $tmp_query);
		$data = pg_fetch_array ($tmp_rs, 0, PGSQL_ASSOC);
		return $data['last_value'];
		//return $this->compat_mode ? pg_getlastoid ($this->rs) : pg_last_oid ($this->rs);
	}

	/**	Return uppercase identifier that this database constants start with.
	 *	Returns "PGSQL" string.
	 *	@access	public
	 */
	function get_db_id () {
		return "PGSQL";
  	}

	/** Returns true if connected to DB.
	 *	// TODO: does not work with compat_mode == true
	 *	@return	boolean
	 *	@access	public
	 */
	function connected () {
		return (is_resource ($this->conn)) &&
			(pg_connection_status ($this->conn) == PGSQL_CONNECTION_OK);
	}

	/**	Select the way the array data is returned.
	 *	Possible solutions are "NUM" for numeric, "ASSOC" for associative and "BOTH".
	 *	Given $type is converted to PGSQL_$type and evaluated.
	 *  @param	$type	one of "NUM", "ASSOC", "BOTH" strings
	 *	@access	public
	 */
	function set_array_return_type ($type="ASSOC") {
		if ($type == "NUM" || $type == "ASSOC" || $type == "BOTH") {
			$this->_array_return_type = $this->$type;
			return true;
		} else
			return false;
  	}

} // class postgres_db

?>
