<?php

/*
 *	Article System
 *	Online journal management tool written in PHP/MySQL/PostgreSQL.
 *	This code is available at http://sourceforge.net/projects/artsys
 *
 *	Copyright (C) 2001-2005 Jan Hnatek
 *	Distributed under the terms of the GNU General Public License
 *
 *	Date:	  05/08/2005
 *	Version:  0.6   	
 */

###############################################################################

/**
 *	Abstract class for an entity in DB.<br>
 *	Used to simplify <code>SELECT</code> SQL queries and rendering of the results.<br><br>
 *
 *	<b>Note: This is Article System specific class. It uses global <code>$db</code> object.</b>
 *
 *	@author	Jan Hnatek
 *	@package db_entities
 */
class db_entity {
//-------------//

	// VARIABLES
	//

	/**	Table name of this entity. */
	var $name;
	/**	Last error string. */
	var $error;
	/**	Query results array. */
	var $result = array();


	// CONSTRUCTOR
	//

	/**	Sets up the <code>db_entity</code> table name.
	 *	@param	$name	Name of the table in DB describing the entity, string.
	 *	@see abstract_db
	 */
	function db_entity ($name) {
		$this->name = $name;
	}


	// METHODS : PUBLIC
	//

	/**	Executes a DB SELECT query.<br>
	 *	Note: this uses <code>abstract_db::get_result_array</code>, so the result array 
	 *	can be either numeric, associative or both.
	 *	@param	$what	string	The text between 'SELECT' and 'FROM'. Column names or "*".
	 *	@param	$where	string	Optional WHERE/ORDER clause.
	 *	@returns array
	 *	@access	public
	 */
	function get ($what, $where="") {
		global $db;
		$this->result = array ();
		$RS = $db->exec ("SELECT $what FROM $this->name $where"); //, 1);
		if (!$RS) {
			$this->error = $db->report (msg ("select fail"));
			return false;
		}
		while ($row = $db->get_result_array ())
			array_push ($this->result, $row);
		return $this->result;
	}

	/**	Executes a DB SELECT query as <code>get</code>, but returns just the first value.
	 *	@param	$what	string	The text between 'SELECT' and 'FROM'. Column names or "*".
	 *	@param	$where	string	Optional WHERE/ORDER clause.
	 *	@returns array
	 *	@access	public
	 */
	function get_one ($what, $where="") {
		if (!$this->get ($what, $where))
			return false;
		return $this->result[0];
	}

	/**	Returns an associative array based on two DB entity columns.
	 *	@param	$ckey	string	First column name. Used as keys to associative array.
	 *	@param	$cval	string	Second column name. Used as values to associative array.
	 *	@param	$assoc	array	An array to put values to.
	 *	@param	$where	string	Optional WHERE/ORDER clause.
	 *	@returns array
	 *	@access	public
	 */
	function get_assoc ($ckey, $cval, &$assoc, $where="") {
		$what = "$ckey, $cval";
		if (!$this->get ($what, $where))
			return false;
		reset ($this->result);
		foreach ($this->result as $row)
			$assoc[$row[$ckey]] = $row[$cval];
		return $assoc;
	}

	/**	Executes a DB DELETE query.
	 *	Returns number of deleted rows.
	 *	@param	$where	string	Compulsory WHERE clause.
	 *	@returns integer
	 *	@access	public
	 */
	function delete ($where) {
		global $db;
		$this->result = array ();
		$RS = $db->exec ("DELETE FROM $this->name $where"); //, 1);
		if (!$RS) {
			$this->error = $db->report (msg ("delete fail"));
			return false;
		}
		return $db->get_affected_rows ();
	}

    /**	Returns text of the last error.
     *	@returns string
     *	@access public
     */
	function get_error () {
		return $this->error;
	}

    /**	Prints formatted error message.
     *	Applications should call this function at a 'placeholder' that displays
	 *	an error message if error occurs.
     *	@access public
     */
	function error_report () {
		if (isset ($this->error)) {
			printf ("<br>%s", $this->error);
			return true;
		} else
			return false;
	}

	/**	Return name of this entity
	 *	@returns string
	 *	@access	public
	 */
	function get_name () {
		return $this->name;
	}

	/**	A HTML form select box renderer.<br>
	 *	Creates a select box based on the results of previous DB query.
	 *	@param	$name	Name of the select form item.
	 *	@param	$cval	Name of column or index to the query result, to be used as option's value="".
	 *	@param	$ctext	Name of column or index to the query result, to be used as option's label (text to display).
	 *	@param	$selected_value
	 *	TODO: ..
	 *	@access	public
	 *	@return	void
	 */
	// it wound be possible to do this using get_assoc and there wound be no need to call ->get explicitly,
	// but like this, it's more common (* can be selected, not just two columns, and can be still worked with) ???
	function select_box ($name, $cval, $ctext, $selected_value, $begin_options="", $end_options="", $attributes="") {
		$sb_array = array ();
		if ($begin_options != "")
			$sb_array = $begin_options;
		reset ($this->result);
		foreach ($this->result as $row)
			$sb_array[$row[$cval]] = $row[$ctext];
		if ($end_options != "")
			$sb_array = array_merge ($sb_array, $end_options);
		render_select_box ($name, $sb_array, $selected_value, $attributes);
	}	// select_box()
}

?>
