<?php

/*
 *	Article System
 *	Online journal management tool written in PHP/MySQL/PostgreSQL.
 *	This code is available at http://sourceforge.net/projects/artsys
 *
 *	Copyright (C) 2001-2005 Jan Hnatek
 *	Distributed under the terms of the GNU General Public License
 *
 *	Date:	  05/08/2005
 *	Version:  0.6   	
 */

###############################################################################
# list_table : a class to render structured data into a table
#

//require_once 'class.tag_writer.php';

/*	HISTORY:
	1.0	/ 12.12.2001
		* "db_table" initial version, used only to render DB results
	1.1	/ 20.3.2002
		* created upper class object "list_table"
		* "db_table" made a subclass of "list_table" with custom constructor ($data structure is created there)
	1.2 / 1.4.2002
		* contructor now accepts the data as reference
	1.3 / 21.7.2002
		* fixed class documentation syntax (php was removing the comments)
	1.4 / 2.11.2002
		* simplified, separated table styling methods into a subclass (_taset)
	1.5 / ?.11.2002
		* more code separated to private methods (useful for subclasses redefinitions)
	1.6 / 12.12.2002
		* removed deprecated methods
	1.7 / 14.12.2002
		* enabled usage of tag_writer class -> tag output class
		* removed custom debugging
        * documentation rewritten
		* more systematic variable/function naming
	1.8 / 2.6.2003, fix 12.6.2003
		* register_user_functions
	TODO:
		* check arrays on input (is_array)
 */

/**
 *	The <code>list_table</code> was written to help rendering of data structured in rows/columns.<br>
 *	Uses {@link tag_writer} to output tags.<br><br>
 *
 *	Features :
 	<ul>
	<li>separately defined header</li>
	<li>periodically applied tr/td attributes (can be used to define switching background color,
		or different field widths)</li>
	<li>fields as checkboxes</li>
	<li>"select all" button</li>
	<li>user defined functions optionally applied to selected fields</li>
	</ul><br>
 *
 *	Note: When using checkboxes with "select_all", the containing form is NOT generated, so you
 *	should give its exact name using <code>set_form_name</code>. Without that, the "select_all" button
 *	will not work.<br><br>
 *
 *	@note Defines special "tr.head" and "tr.foot" parameters for <code>tag_writer</code> subclasses.<br>
 *
 *	@see		tag_writer
 *	@author		Jan Hnatek
 *	@package lists
 */
class list_table {
//--------------//

	// VARIABLES
	//

	/**	Table data. */
	var $_data;

	/**	Number of table rows. */
	var $_rows;

	/**	Number of table columns. */
	var $_cols;

	/**	Name of the form, that this table will be part of. */
	var $_form_name;

	/**	Associated <code>tag_writer</code>. */
	var $_tw;

	// features enable/disable
	var $_draw_header;
    var $_draw_checkboxes;
    var $_draw_select_all;
	var $_draw_boolean;

	// features data storage
	var $_header_cols;
	var $_hide_cols;
    var $_chbox_cols;
	var $_chbox_prefix;
	var $_select_all_text;
	var $_boolean_cols;
	var $_boolean_true;
	var $_boolean_false;

	// user functions
	var	$_user_functions;
	var $_debug_user_fn;


	// CONSTRUCTOR
	//

	/** Constructor for <code>list_table</code>.
	 *	@param	&$data		reference to a 2D array of data (not associative) to render
	 *	@param	&$twriter	reference to <code>tag_writer</code> class instance
	 *	@return	boolean
	 *	@access	public	 
	 */
    function list_table (&$data, &$twriter) {
		$this->_tw = $twriter;
		if (!$this->set_data ($data)) {
			echo "list_table: input data empty\n";
			return false;
		}
		$this->_header_cols = array ();
		$this->_hide_cols = array ();
		$this->_draw_header = false;
    	$this->_draw_checkboxes = false;
    	$this->_draw_select_all = false;
		$this->_draw_boolean = false;
		$this->_form_name = "item_list";
		$this->_user_functions = array ();
		$this->_debug_user_fn = false;
	}


	// PUBLIC METHODS
	//

	/**	Set table header data (first line of the table)
	 *	@param	$column_array	array of column names
	 *	@access	public
	 */
	function use_header ($column_array) {
		$this->_draw_header = true;
		$this->_header_cols = $column_array;
	}

	/**	Render specified columns as checkboxes
	 *	@param	$column_numbers		array of column numbers
	 *	@param	$name_prefix		prefix string to use in name attribute of the input tag (name="prefix . data[i][j]")
	 *	@access	public
	 */
	function use_checkboxes ($column_numbers, $name_prefix="X") {
		$this->_draw_checkboxes = true;
		$this->_chbox_cols = $column_numbers;
		$this->_chbox_prefix = $name_prefix;
	}

	/**	Enable "select all" button (checkbox) in the footer of the table
	 *	@param	$name	text to be printed next to the button
	 *	@access	public
	 */
	function use_select_all ($name="select all") {
		$this->_draw_select_all = true;
		$this->_select_all_text = $name;
	}

	/**	Render specified columns as (user defined) boolean values.<br>
	 *	@param	$column_numbers		array of column numbers
	 *	@param	$true_val			true value string (may be an img tag)
	 *	@param	$false_val			false value string (may be an img tag)
	 *	@access	public
	 */
	function use_boolean ($column_numbers, $true_val="1", $false_val="0") {
		$this->_draw_boolean = true;
		$this->_boolean_cols = $column_numbers;
		$this->_boolean_true = $true_val;
		$this->_boolean_false = $false_val;
	}

	/** Specify the set of user defined functions, that will be applied
	 *	to specified columns.
	 *	Special values may be passed as arguments: <br>
		array (	"/%DATA%/",
				"/%NROWS%/",
				"/%NCOLS%/",
				"/%RNUM%/",
				"/%CNUM%/")
	 *	@param	$user_fn	Array: column numbers as keys, function calls as textual values.
	 *	@param	$debug_mode	Boolean to enable eval call debugging (often a cause of syntax errors).
	 *	@access	public
	 */
	function register_user_functions ($user_fn, $debug_mode) {
	 	if (is_array ($user_fn))
			$this->_user_functions = $user_fn;
		$this->_debug_user_fn = $debug_mode;
	}
	
	/**	Set columns that won't be displayed
	 *	@param	$column_array	array of column numbers or names
	 *	@access	public
	 */
	function hide_columns ($column_array) {
		$this->_hide_cols = $column_array;
	}

	/**	Table rendering function. This method creates the final table code.
	 *	Consists of private methods for every task for easy overriding in subclasses.
	 *	@access	public
	 */
	function display () {
		$this->_prerender_check ();
		$this->_table_begin ();
		$this->_table_head ();
		$this->_table_rows ();
		$this->_table_foot ();
		$this->_table_end ();
	}

	/**	Set data source for this table.
	 *	A two-dimensional array is expected.
	 *	Number of columns is derived from the size of the first row.
	 *	Returns false if input data is empty
	 *	@param	&$data		reference to a 2D array.
	 *	@return	boolean
	 *	@access	public
	 */
	function set_data (&$data) {
		if (count ($data)) {
			$this->_data = $data;
			$this->_rows = count ($this->_data);
			$this->_cols = isset ($this->_data[0]) ? count ($this->_data[0]) : -1;
			//debug ("list_table: rows: ".$this->_rows);
			//debug ("list_table: columns: ".$this->_cols);
			return true;
		} else {
			return false;
		}
	}

    /**	Return data array from the table.
	 *	@return	2D array
	 *	@access	public
	 */
	function get_data () {
		return $this->_data;
	}

	/**	Set the name of containing form.
	 *	@access	public
	 */
	function set_form_name ($name) {
		$this->_form_name = $name;
	}

	/**	Get the name of containing form.
	 *	@access	public
	 */
	function get_form_name () {
		return $this->_form_name;
	}



		// RENDERING TOOLS : PRIVATE
		//

		/**	Check, that "select_all" is enabled only when checkboxes are used.
		 *	@access	private
		 */
		function _prerender_check () {
			if ($this->_draw_select_all && empty ($this->_chbox_cols)) {
				warning ("list_table::_prerender_check: you did not specify where to use the checkboxes,
				use <code>use_checkboxes()</code> method");
			}
		}

		/**	Render table begin.
		 *	@access	private
		 */
		function _table_begin () {
			// create table start
			echo "\n\n<!-- list_table : generated output : begin -->\n";
			$this->_tw->begin ("table");
		}

		/**	Render table head. Special "tr.head" parameter is set with $lt;tr&gt; tag.
		 *	@access	private
		 */
		function _table_head () {
			// table head
			if ($this->_draw_header) {
				// set a special parameter with "tr.head" table row ..
				$this->_tw->begin ("thead", array ("tr", "tr.head"));
				for ($i=0; $i < $this->_cols; $i++) {
					// use strict in_array() .. true
					if (!in_array ($i, $this->_hide_cols, true) &&
						!in_array ($this->_header_cols[$i], $this->_hide_cols)) {
						$col = isset ($this->_header_cols[$i]) ? $this->_header_cols[$i] : "";
						$this->_tw->full ("td", $col);
					}
				}
				$this->_tw->end ("tr", "thead");
			}
		}

		/**	Render table content - rows - from data.
		 *	@access	private
		 */
		function _table_rows () {
			for ($i=0; $i < $this->_rows; $i++)
				$this->_table_row ($i);
		}

		/**	Render one table row.
		 *	@param	$i	Data index (global row number).
		 *	@access	private
		 */
		function _table_row ($i) {
			$this->_tw->begin ("tr");
			for ($j=0; $j < $this->_cols; $j++) {
				// use strict in_array() .. true
				if (!in_array ($j, $this->_hide_cols, true) &&
					!in_array ($this->_header_cols[$j], $this->_hide_cols)) {
					$cell = $this->_data[$i][$j];
					if (($this->_draw_checkboxes) && (in_array ($j, $this->_chbox_cols))) {
						// render checkbox
						$this->_tw->begin ("td");
						?><input type="checkbox" value="<?= $cell?>" name="<?= $this->_chbox_prefix.$cell?>"><?
						$this->_tw->end ("td");
					// TODO: remove this? .. left here for compatibility {
					} elseif (($this->_draw_boolean) && (in_array ($j, $this->_boolean_cols))) {
						// render boolean
						$cell = $cell ? $this->_boolean_true : $this->_boolean_false;
						$this->_tw->full ("td", $cell);
					// }
					} elseif (in_array ($j, array_keys ($this->_user_functions, true))) {
						$funcs = $this->_user_functions[$j];
						if (!is_array ($funcs))
							$funcs = array ($funcs);
						foreach ($funcs as $func) {
							$func = preg_replace (
								array (	"/%DATA%/",
										"/%NROWS%/",
										"/%NCOLS%/",
										"/%RNUM%/",
										"/%CNUM%/",
								),
								array (	addslashes ($cell),
										$this->_rows,
										$this->_cols,
										$i,
										$j,
								), $func);
							while (preg_match ("/%DATA[0-9]*%/", $func)) {
								$colnum = preg_replace ("/.*%DATA([0-9]*)%.*/", "$1", $func);
								$func = preg_replace ("/%DATA[0-9]*%/", $this->_data[$i][$colnum], $func);
							}
							if ($this->_debug_user_fn)
								debug ($func);
							eval ("\$cell = $func;");
						}
						$this->_tw->full ("td", $cell);
					} else {
						// render normally
						$this->_tw->full ("td", $cell);
					}
				}
			}
			$this->_tw->end ("tr");
		}

		/**	Render table footer. Optional "select_all" checkbox.
		 *	Special "tr.foot" parameter is set with $lt;tr&gt; tag.
		 *	@access	private
		 */
		function _table_foot () {
			// optional select_all footer
			if ($this->_draw_select_all && $this->_rows) {
				$size = $this->_cols - 1;
				// set a special parameter with "tr.foot" table row ..
				$this->_tw->begin (array ("tr", "tr.foot"), array ("td", "td.foot"));
				?><input type="checkbox" name="select_all" onclick="select_all_<?= $this->_form_name?> ()"><?
				$this->_tw->end ("td");
				$this->_tw->open (array("td", "td.foot"));
                echo " colspan=\"$size\">". $this->_select_all_text;
				$this->_tw->end ("td", "tr");
			}
		}

		/**	Render table end. Optional output of "select_all" (java)script.
		 *	@access	private
		 */
		function _table_end () {
			// finalize
			echo "</table>\n";
			if ($this->_draw_select_all) {
				echo "\n<!-- list_table : generated output : script -->\n\n";
				$this->_script_select_all ();
			}
			echo "\n<!-- list_table : generated output : end -->\n\n";
		}

		/**	Output javascript controlling "select_all" checkbox.
		 *	@access	private
		 */
		function _script_select_all () {	?>
		<script language="JavaScript" type="text/javascript">
		<!--
		function select_all_<?= $this->_form_name?> () {
			var state = document.<?= $this->_form_name?>.select_all.checked;
			for (var i=0; i < document.<?= $this->_form_name?>.elements.length-1; i++)
				if (document.<?= $this->_form_name?>.elements[i].checked != state)
					document.<?= $this->_form_name?>.elements[i].click ();
		}
		-->
		</script>
<?		}


	// TABLE DATA RENDERERS : PUBLIC
	// used with array passed to $this->register_user_functions ()
	//

	/**	Render cell as boolean, given true and false value.
	 *	ex: "list_table::to_boolean (%DATA%, \"<b>yes</b>\", \"no\")"
	 *	@access	public
	 */
	function to_boolean ($cell_data, $true_val, $false_val) {
		if (!is_numeric ($cell_data))
			warning ("not a numeric value");
		return $cell_data ? $true_val : $false_val;
	}

	/**	Render cell as link (anchor tag), given text, url and id.<br>
	 *	note: final url is rendered as "$url$id", so take care of ?/&
	 *	ex: "to_link (\"%DATA%\", \"main.php?id=\", %RNUM%)"
	 *	@access	public
	 */
	function to_link ($text, $url, $id) {
		return "<a href=\"$url$id\">$text</a>";
	}
	
	/**	Concatenate existing cell value with given string and save it as new cell value.
	 *	note: final url is rendered as "$url$id", so take care of ?/&
	 *	ex: "concat (\"%DATA%\", \"some_information\")"
	 *	@access	public
	 */
	function concat ($old, $add) {
		return "$old$add";
	}
	
}	// class list_table

?>
