<?php

/*
 *	Article System
 *	Online journal management tool written in PHP/MySQL/PostgreSQL.
 *	This code is available at http://sourceforge.net/projects/artsys
 *
 *	Copyright (C) 2001-2005 Jan Hnatek
 *	Distributed under the terms of the GNU General Public License
 *
 *	Date:	  05/08/2005
 *	Version:  0.6   	
 */

###############################################################################

/**
 *	Logging system.<br>
 *  Handle all messages according to debug level. Can be shown onscreen,
 *	written to file, sent to email, debugger or system log.
 *
 *	@author	Jan Hnatek
 *	@package log
 */

	// SOME CONSTANTS
	//

	// logging destinations
	define ("MLOG_TRASH",		0);
	define ("MLOG_SCREEN",		1);
	define ("MLOG_FILE", 		2);
	define ("MLOG_SYSLOG",		4);
	define ("MLOG_EMAIL",		8);
	define ("MLOG_DEBUG_HOST",	16);

	// message types and/or debug level parts
	define ("MLOG_NONE",				0);
	define ("MLOG_DEBUG",				1);
	define ("MLOG_DEBUG_DB",			2);
	define ("MLOG_DEBUG_DB_MASSIVE",	4);
	define ("MLOG_NOTICE",	E_USER_NOTICE);		// 1024
	define ("MLOG_WARNING",	E_USER_WARNING);	// 512
	define ("MLOG_ERROR",	E_USER_ERROR);		// 256

	// debug level presets
	define ("MLOG_DEFAULT",	MLOG_NOTICE | MLOG_WARNING | MLOG_ERROR);
	define ("MLOG_ALL",		MLOG_NOTICE | MLOG_WARNING | MLOG_ERROR |
							MLOG_DEBUG | MLOG_DEBUG_DB);
	define ("MLOG_MASSIVE",	MLOG_ALL | MLOG_DEBUG_DB_MASSIVE);


class mlog {
//--------//

	// VARIABLES
	//

	// debug/message level
	var $level;
	// use external message !screen! displaying functions -> trigger_error()
	var $external;
	// destinations
	var $destinations;

	// destination specifics
	var $logfile='';
	var $lfp;
	var $email='';
	var $syslog;
	var $host='';
	var $extra_headers='';


	// CONSTRUCTOR
	//

	/**	Set logging defaults - destination is screen for all messages.<br>
	 *	@param	$level	Debug level to use to handle messages.<br>
	 *			Use predefined constants to set this.
	 *	@param	$use_trigger_error	Enable use of PHPs <code>trigger_error()</code>
	 *			function to display messages. The behaviour of output methods depends
	 *			on their implementation (they may not take this option into account).
	 *	@return void
	 *	@access	public
	 */
	function mlog ($level=MLOG_DEFAULT, $use_trigger_error=false) {
		$this->level = $level;
		$this->external = $use_trigger_error;
		$this->destinations = array (
			MLOG_NONE		=> MLOG_TRASH,
			MLOG_DEBUG		=> MLOG_SCREEN,
			MLOG_DEBUG_DB	=> MLOG_SCREEN,
			MLOG_DEBUG_DB_MASSIVE	=> MLOG_SCREEN,
			MLOG_NOTICE 	=> MLOG_SCREEN,
			MLOG_WARNING	=> MLOG_SCREEN,
			MLOG_ERROR		=> MLOG_SCREEN,
		);
	}


	// SETUP METHODS
	//

	/**	Set destination for given type of messages.
	 *	@param	$type	Message type to set up.
	 *	@param	$dest	Places where to put this message.
	 *	@return void
	 *	@access	public
	 */
	function set_destination ($type, $dest) {
		if (!in_array ($type, array_keys ($this->destinations)))
			user_error ("log: unknown message type", MLOG_ERROR);
        else
			$this->destinations[$type] = $dest;
	}

	/**	Set usage of log file (if given filename is not empty).
	 *	Optionally, you can specify what types of messages will be sent there.
	 *	@param	$filename	Name of existing and writable file.
	 *	@param	$types		Array of message types to log to file.
	 *	@return boolean
	 *	@access	public
	 */
	function set_logfile ($filename, $types='') {
		$this->logfile = $filename;
		if (!empty ($filename)) {
			if (!file_exists ($filename)) {
				user_error ("log: logfile does not exist", MLOG_ERROR);
				return false;
    		}
    		if (!is_writable ($filename)) {
				user_error ("log: logfile is not writable", MLOG_ERROR);
				return false;
    		}
			$this->lfp = fopen ($this->logfile, "a");
			if (!$this->lfp) {
				user_error ("log: unable to open logfile", MLOG_ERROR);
				return false;
			}
			if (is_array ($types))
				foreach ($types as $type)
					$this->destinations[$type] |= MLOG_FILE;
		}
		return true;
	}
	function set_email ($email, $extra_headers, $types='') {
		$this->email = $email;
		$this->extra_headers = $extra_headers;
		if (is_array ($types))
			foreach ($types as $type)
				$this->destinations[$type] |= MLOG_EMAIL;
	}
	function set_syslog ($types='') {
		$this->syslog = true;
		if (is_array ($types))
			foreach ($types as $type)
				$this->destinations[$type] |= MLOG_SYSLOG;
	}
	function set_debug_host ($host, $types='') {
		$this->host = $host;
		if (is_array ($types))
			foreach ($types as $type)
				$this->destinations[$type] |= MLOG_DEBUG_HOST;
	}


	// MESSAGE DISTRIBUTION METHODS
	//

	/**	Distribute given message to all registered destinations for given type.
	 *	@param	$text	Message text.
	 *	@param	$type	Type of the message (MLOG_{NOTICE, WARNING, ERROR, DEBUG..}
	 *	@access	public
	 *	@return	boolean
	 */
	function message ($text, $type) {
		// check that the type is correct
		if (!in_array ($type, array_keys ($this->destinations))) {
			user_error ("log: unknown message type", MLOG_ERROR);
			return false;
	   	}
		// if current log level contains this type of message
		if (binary_isset ($this->level, $type)) {
			// get list of destinations
			$mydest = $this->destinations[$type];
	   		// write to each destination if set
			if (binary_isset ($mydest, MLOG_SCREEN))
				$this->_render_screen ($text, $type);
			if (binary_isset ($mydest, MLOG_FILE))
				$this->_render_file ($text, $type);
			if (binary_isset ($mydest, MLOG_SYSLOG))
				$this->_render_syslog ($text, $type);
			if (binary_isset ($mydest, MLOG_EMAIL))
				$this->_render_email ($text, $type);
    		if (binary_isset ($mydest, MLOG_DEBUG_HOST))
				$this->_render_debug_host ($text, $type);
    	}
	}


	// PRIVATE : MESSAGE RENDERING METHODS : ABSTRACT
	//

	/**	Render message on screen, according to message type.
	 *	@param	$text	Message text.
	 *	@param	$type	Type of the message (MLOG_{NOTICE, WARNING, ERROR, DEBUG..}
	 *	@access	private
	 */
	function _render_screen ($text, $type) {
		// abstract
		/*	example:
   		if ($this->external)
			user_error ($text, $type);
		else
			echo "<p class=\"warning\">$text</p>\n"; */
  	}

	/**	Render message and write to logfile, according to message type.
	 *	@param	$text	Message text.
	 *	@param	$type	Type of the message (MLOG_{NOTICE, WARNING, ERROR, DEBUG..}
	 *	@access	private
	 */
	function _render_file ($text, $type) {
		// abstract
  	}

	/**	Render message and write to syslog, according to message type.
	 *	@param	$text	Message text.
	 *	@param	$type	Type of the message (MLOG_{NOTICE, WARNING, ERROR, DEBUG..}
	 *	@access	private
	 */
	function _render_syslog ($text, $type) {
		// abstract
	}

	/**	Render message and send to email, according to message type.
	 *	@param	$text	Message text.
	 *	@param	$type	Type of the message (MLOG_{NOTICE, WARNING, ERROR, DEBUG..}
	 *	@access	private
	 */
	function _render_email ($text, $type) {
		// abstract
	}

	/**	Render message and send to debug host, according to message type.
	 *	@param	$text	Message text.
	 *	@param	$type	Type of the message (MLOG_{NOTICE, WARNING, ERROR, DEBUG..}
	 *	@access	private
	 */
	function _render_debug_host ($text, $type) {
		// abstract
	}

}	// class mlog

/* destinations : global for every message class
	-> screen
	-> syslog
	-> file
	-> debugging host
	-> email
* message options
	-> no remote - debugging passwords ?
	-> optional debug (
*/

?>
